@@ -31,6 +31,7 @@
  * 19 Sep 01, tuorfa@yahoo.com: added output personality
  * 29 Mar 05, daved@physiol.usyd.edu.au: changes requested by ZT Smith
  * 16 Dec 07, daved@physiol.usyd.edu.au: updated to GPL v3
+ * 09 Nov 08, arkadiusz.firus@gmail.com: define CONFIG_DIR
  *--------------------------------------------------------------------*/
 
 
@@ -45,6 +46,9 @@ extern int no_remap_mode;
 #include "output.h"
 #endif
 
+#define CONFIG_DIR "/usr/local/lib/unrtf/outputs/"
+#define DEFAULT_OUTPUT "html"
+
 extern OutputPersonality *op;
 
 
