@@ -36,6 +36,7 @@
  * 16 Dec 07, daved@physiol.usyd.edu.au: updated to GPL v3
  * 17 Dec 07, daved@physiol.usyd.edu.au: added support for --noremap from
  *		David Santinoli
+ * 09 Nov 08, arkadiusz.firus@gmail.com: use iconv
  *--------------------------------------------------------------------*/
 
 
@@ -63,6 +64,11 @@
 #include "convert.h"
 
 
+#ifndef HAVE_ATTR_H
+#include "attr.h"
+#define HAVE_ATTR_H
+#endif
+
 /*========================================================================
  * Name:	op_create
  * Purpose:	Creates a blank output personality.
@@ -83,8 +89,6 @@ op_create ()
 	return new_op;
 }
 
-
-
 /*========================================================================
  * Name:	op_free
  * Purpose:	Deallocates an output personality, but none of the strings
@@ -113,103 +117,97 @@ op_free (OutputPersonality *op)
  *=======================================================================*/
 
 char *
-#if 1 /* daved - 0.19.6 */
-op_translate_char (OutputPersonality *op, int charset, CodepageInfo *codepage, int ch, int ntable)
-#else
-op_translate_char (OutputPersonality *op, int charset, CodepageInfo *codepage, int ch)
-#endif
+op_translate_char (OutputPersonality *op, my_iconv_t cd, int ch)
 {
 	short start;
 	char *result=NULL;
-#if 1	/* daved - 0.20.5 */
 	static char output_buffer[2]={ 0, 0 };
-#endif
+	char *inbuf, *outbuf;
+	size_t inbytes = (ch / 256) + 1, outbytes = inbytes * 4, i;
 
 	CHECK_PARAM_NOT_NULL(op);
 
-#if 1	/* daved - 0.20.5 */
 	if (no_remap_mode == TRUE && ch < 256)
 	{
 		output_buffer[0]=ch;
 		result=output_buffer;
 	}
 	else
-#endif
-#if 1 /* daved - 0.19.6 */
-	/* if we are seeking a character from a symbol font we can
-	   be below 0x80
-	*/
-	if(ntable == FONTSYMBOL_TABLE)
+	if (result == NULL)
 	{
-		start = op->symbol_first_char;
+		inbuf = my_malloc(inbytes + 1);
+		outbuf = my_malloc(outbytes + 1);
 
-		if(ch >= start && ch <= op->symbol_last_char)
-			result = op->symbol_translation_table[ch - start];
-		if(result)
-			return result;
-	}
-	else
-#endif
-#if 1 /* daved - 0.20.3 */
-	if(ntable == FONTGREEK_TABLE)
-	{
-		start = op->greek_first_char;
-
-		if(ch >= start && ch <= op->greek_last_char)
-			result = op->greek_translation_table[ch - start];
-		if(result)
-			return result;
-	}
-#endif
-	if (ch >= 0x20 && ch < 0x80) {
-		result = op->ascii_translation_table [ch - 0x20];
-	}
-	else
-	if (charset != CHARSET_ANSI &&
-	    charset != CHARSET_MAC &&
-	    charset != CHARSET_CP437 &&
-	    charset != CHARSET_CP850)
-		error_handler ("invalid character set value, cannot translate character");
-	else
-	switch (charset) {
-	case CHARSET_ANSI:
-		if (codepage != NULL && op->unisymbol_print != NULL && codepage->cp)
+		for (i = inbytes - 1; ch > 255; i--)
 		{
-			if(0)
-			printf("<CODEPAGE CHAR %d>", codepage->chars[ch - 0x80]);
-			if (codepage->chars[ch - 0x80]) {
-				if(0)
-				printf("<UNIPRINTING>");
-				result = op->unisymbol_print(codepage->chars[ch - 0x80]);
+			inbuf[i] = ch % 256;
+			ch /= 256;
+		}
+
+		inbuf[0] = ch;
+		inbuf[inbytes] = '\0';
+		i = outbytes;
+		if (!my_iconv_is_valid(cd))
+		{
+			cd = my_iconv_open("UTF-8", "cp1252");
+
+			if (my_iconv(cd, &inbuf, &inbytes, &outbuf, &outbytes) == -1)
+			{
+				fprintf(stderr, "unrtf: Error in executing iconv1\n");
+				return NULL;
 			}
+			
+			my_iconv_close(cd);			
 		}
-		if(!result)
+		else
+
+			if (my_iconv(cd, &inbuf, &inbytes, &outbuf, &outbytes) == -1)
+			{
+				fprintf(stderr, "unrtf: Error in executing iconv\n");
+				return NULL;
+			}
+		*outbuf = '\0';
+		outbuf -= i - outbytes;
+/* Conversion from string to utf8 code number */
+		inbytes = 0;
+
+		for (i = 0; outbuf[i] != '\0'; i++)
+			inbytes++;
+
+		ch = 0;
+
+		for (i = 0; i < inbytes; i++)
 		{
-		start = op->ansi_first_char;
-			if (ch >= start &&
-			    ch <= op->ansi_last_char)
-				result = op->ansi_translation_table [ch-start];
+			if (i == 0)
+				switch (inbytes)
+				{
+					case 1:
+						ch =  outbuf[0];
+						break;
+					case 2:
+						ch = (unsigned char) outbuf[0] - 192;
+						break;
+					case 3:
+						ch = (unsigned char) outbuf[0] - 224;
+						break;
+					case 4:
+						ch = (unsigned char) outbuf[0] - 240;
+						break;
+				}
+			else
+				ch = (ch * 64) + ((unsigned char) outbuf[i] - 128);
 		}
-		break;
-	case CHARSET_MAC:
-		start = op->mac_first_char;
-		if (ch >= start &&
-		    ch <= op->mac_last_char)
-			result = op->mac_translation_table [ch-start];
-		break;
-	case CHARSET_CP437:
-		start = op->cp437_first_char;
-		if (ch >= start &&
-		    ch <= op->cp437_last_char)
-			result = op->cp437_translation_table [ch-start];
-		break;
-	case CHARSET_CP850:
-		start = op->cp850_first_char;
-		if (ch >= start &&
-		    ch <= op->cp850_last_char)
-			result = op->cp850_translation_table [ch-start];
-		break;
+/* End of conversion*/
+
+		result = get_alias(op, ch);
+
+		if (result == NULL)
+			if (ch > 127 && op->unisymbol_print)
+				result = assemble_string(op->unisymbol_print, ch);
+			else
+				result = outbuf;
 	}
+
 	return result;
 }
 
@@ -234,49 +232,49 @@ op_begin_std_fontsize (OutputPersonality *op, int size)
 	switch (size) {
 	case 8:
 		if (op->fontsize8_begin) {
-			printf (op->fontsize8_begin);
+			if (safe_printf(0, op->fontsize8_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize8_begin");
 			found_std_expr = TRUE;
 		}
 		break;
 	case 10:
 		if (op->fontsize10_begin) {
-			printf (op->fontsize10_begin);
+			if (safe_printf(0, op->fontsize10_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize10_begin");
 			found_std_expr = TRUE;
 		}
 		break;
 	case 12:
 		if (op->fontsize12_begin) {
-			printf (op->fontsize12_begin);
+			if (safe_printf(0, op->fontsize12_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize12_begin");
 			found_std_expr = TRUE;
 		}
 		break;
 	case 14:
 		if (op->fontsize14_begin) {
-			printf (op->fontsize14_begin);
+			if (safe_printf(0, op->fontsize14_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize14_begin");
 			found_std_expr = TRUE;
 		}
 		break;
 	case 18:
 		if (op->fontsize18_begin) {
-			printf (op->fontsize18_begin);
+			if (safe_printf(0, op->fontsize18_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize18_begin");
 			found_std_expr = TRUE;
 		}
 		break;
 	case 24:
 		if (op->fontsize24_begin) {
-			printf (op->fontsize24_begin);
+			if (safe_printf(0, op->fontsize24_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize24_begin");
 			found_std_expr = TRUE;
 		}
 		break;
 	case 36:
 		if (op->fontsize36_begin) {
-			printf (op->fontsize36_begin);
+			if (safe_printf(0, op->fontsize36_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize36_begin");
 			found_std_expr = TRUE;
 		}
 		break;
 	case 48:
 		if (op->fontsize48_begin) {
-			printf (op->fontsize48_begin);
+			if (safe_printf(0, op->fontsize48_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize48_begin");
 			found_std_expr = TRUE;
 		}
 		break;
@@ -289,53 +287,53 @@ op_begin_std_fontsize (OutputPersonality *op, int size)
 		if (op->fontsize_begin) {
 			char expr[16];
 			sprintf (expr, "%d", size);
-			printf (op->fontsize_begin, expr);
+			if (safe_printf (1, op->fontsize_begin, expr)) fprintf(stderr, TOO_MANY_ARGS, "fontsize_begin");
 		} else {
 			/* If we cannot write out a change for the exact
 			 * point size, we must approximate to a standard
 			 * size.
 			 */
 			if (size<9 && op->fontsize8_begin) {
-				printf (op->fontsize8_begin);
+				if (safe_printf(0, op->fontsize8_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize8_begin");
 			} else 
 			if (size<11 && op->fontsize10_begin) {
-				printf (op->fontsize10_begin);
+				if (safe_printf(0, op->fontsize10_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize10_begin");
 			} else 
 			if (size<13 && op->fontsize12_begin) {
-				printf (op->fontsize12_begin);
+				if (safe_printf(0, op->fontsize12_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize12_begin");
 			} else 
 			if (size<16 && op->fontsize14_begin) {
-				printf (op->fontsize14_begin);
+				if (safe_printf(0, op->fontsize14_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize14_begin");
 			} else 
 			if (size<21 && op->fontsize18_begin) {
-				printf (op->fontsize18_begin);
+				if (safe_printf(0, op->fontsize18_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize18_begin");
 			} else 
 			if (size<30 && op->fontsize24_begin) {
-				printf (op->fontsize24_begin);
+				if (safe_printf(0, op->fontsize24_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize24_begin");
 			} else 
 			if (size<42 && op->fontsize36_begin) {
-				printf (op->fontsize36_begin);
+				if (safe_printf(0, op->fontsize36_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize36_begin");
 			} else 
 			if (size>40 && op->fontsize48_begin) {
-				printf (op->fontsize48_begin);
+				if (safe_printf(0, op->fontsize48_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize48_begin");
 			} else 
 			/* If we can't even produce a good approximation,
 			 * just try to get a font size near 12 point.
 			 */
 			if (op->fontsize12_begin)
-				printf (op->fontsize12_begin);
+				if (safe_printf(0, op->fontsize12_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize12_begin");
 			else
 			if (op->fontsize14_begin)
-				printf (op->fontsize14_begin);
+				if (safe_printf(0, op->fontsize14_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize14_begin");
 			else
 			if (op->fontsize10_begin)
-				printf (op->fontsize10_begin);
+				if (safe_printf(0, op->fontsize10_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize10_begin");
 			else
 			if (op->fontsize18_begin)
-				printf (op->fontsize18_begin);
+				if (safe_printf(0, op->fontsize18_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize18_begin");
 			else
 			if (op->fontsize8_begin)
-				printf (op->fontsize8_begin);
+				if (safe_printf(0, op->fontsize8_begin)) fprintf(stderr, TOO_MANY_ARGS, "fontsize8_begin");
 			else
 				error_handler ("output personality lacks sufficient font size change capability");
 		}
@@ -363,49 +361,49 @@ op_end_std_fontsize (OutputPersonality *op, int size)
 	switch (size) {
 	case 8:
 		if (op->fontsize8_end) {
-			printf (op->fontsize8_end);
+			if (safe_printf(0, op->fontsize8_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize8_end");
 			found_std_expr = TRUE;
 		}
 		break;
 	case 10:
 		if (op->fontsize10_end) {
-			printf (op->fontsize10_end);
+			if (safe_printf(0, op->fontsize10_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize10_end");
 			found_std_expr = TRUE;
 		}
 		break;
 	case 12:
 		if (op->fontsize12_end) {
-			printf (op->fontsize12_end);
+			if (safe_printf(0, op->fontsize12_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize12_end");
 			found_std_expr = TRUE;
 		}
 		break;
 	case 14:
 		if (op->fontsize14_end) {
-			printf (op->fontsize14_end);
+			if (safe_printf(0, op->fontsize14_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize14_end");
 			found_std_expr = TRUE;
 		}
 		break;
 	case 18:
 		if (op->fontsize18_end) {
-			printf (op->fontsize18_end);
+			if (safe_printf(0, op->fontsize18_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize18_end");
 			found_std_expr = TRUE;
 		}
 		break;
 	case 24:
 		if (op->fontsize24_end) {
-			printf (op->fontsize24_end);
+			if (safe_printf(0, op->fontsize24_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize24_end");
 			found_std_expr = TRUE;
 		}
 		break;
 	case 36:
 		if (op->fontsize36_end) {
-			printf (op->fontsize36_end);
+			if (safe_printf(0, op->fontsize36_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize36_end");
 			found_std_expr = TRUE;
 		}
 		break;
 	case 48:
 		if (op->fontsize48_end) {
-			printf (op->fontsize48_end);
+			if (safe_printf(0, op->fontsize48_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize48_end");
 			found_std_expr = TRUE;
 		}
 		break;
@@ -418,57 +416,84 @@ op_end_std_fontsize (OutputPersonality *op, int size)
 		if (op->fontsize_end) {
 			char expr[16];
 			sprintf (expr, "%d", size);
-			printf (op->fontsize_end, expr);
+			if (safe_printf(1, op->fontsize_end, expr)) fprintf(stderr, TOO_MANY_ARGS, "fontsize_end");
 		} else {
 			/* If we cannot write out a change for the exact
 			 * point size, we must approximate to a standard
 			 * size.
 			 */
 			if (size<9 && op->fontsize8_end) {
-				printf (op->fontsize8_end);
+				if (safe_printf(0, op->fontsize8_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize8_end");
 			} else 
 			if (size<11 && op->fontsize10_end) {
-				printf (op->fontsize10_end);
+				if (safe_printf(0, op->fontsize10_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize10_end");
 			} else 
 			if (size<13 && op->fontsize12_end) {
-				printf (op->fontsize12_end);
+				if (safe_printf(0, op->fontsize12_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize12_end");
 			} else 
 			if (size<16 && op->fontsize14_end) {
-				printf (op->fontsize14_end);
+				if (safe_printf(0, op->fontsize14_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize14_end");
 			} else 
 			if (size<21 && op->fontsize18_end) {
-				printf (op->fontsize18_end);
+				if (safe_printf(0, op->fontsize18_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize18_end");
 			} else 
 			if (size<30 && op->fontsize24_end) {
-				printf (op->fontsize24_end);
+				if (safe_printf(0, op->fontsize24_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize24_end");
 			} else 
 			if (size<42 && op->fontsize36_end) {
-				printf (op->fontsize36_end);
+				if (safe_printf(0, op->fontsize36_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize36_end");
 			} else 
 			if (size>40 && op->fontsize48_end) {
-				printf (op->fontsize48_end);
+				if (safe_printf(0, op->fontsize48_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize48_end");
 			} else 
 			/* If we can't even produce a good approximation,
 			 * just try to get a font size near 12 point.
 			 */
 			if (op->fontsize12_end)
-				printf (op->fontsize12_end);
+				if (safe_printf(0, op->fontsize12_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize12_end");
 			else
 			if (op->fontsize14_end)
-				printf (op->fontsize14_end);
+				if (safe_printf(0, op->fontsize14_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize14_end");
 			else
 			if (op->fontsize10_end)
-				printf (op->fontsize10_end);
+				if (safe_printf(0, op->fontsize10_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize10_end");
 			else
 			if (op->fontsize18_end)
-				printf (op->fontsize18_end);
+				if (safe_printf(0, op->fontsize18_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize18_end");
 			else
 			if (op->fontsize8_end)
-				printf (op->fontsize8_end);
+				if (safe_printf(0, op->fontsize8_end)) fprintf(stderr, TOO_MANY_ARGS, "fontsize8_end");
 			else
 				error_handler ("output personality lacks sufficient font size change capability");
 		}
 	}
 }
 
+#if 1 /* AK3 - AF */
+/*========================================================================
+ * Name:	add_alias
+ * Purpose:	Adds alias (text) for a char number nr.
+ * Args:	OutputPersonality, char's number, alias.
+ * Returns:	None.
+ *=======================================================================*/
+
+void
+add_alias(OutputPersonality *op, int nr, char *text)
+{
+	op->aliases = (Aliases *)add_to_collection((Collection *)op->aliases, nr, text);
+}
+
+/*========================================================================
+ * Name:	get_alias
+ * Purpose:	Search for alias for character number nr.
+ * Args:	OutputPersonality, char's number.
+ * Returns:	Text alias or NULL if found nothing.
+ *=======================================================================*/
+
+char *
+get_alias(OutputPersonality *op, int nr)
+{
+	return get_from_collection((Collection *)op->aliases, nr);
+}
+#endif
 
